#!/usr/bin/bash
[ -r "$HOME/.docker-runner" ] && source "$HOME/.docker-runner"
[ -r ".docker-runner" ] && source ".docker-runner"

#------------------------------------------------------------------------------
# 1. 环境变量:
#------------------------------------------------------------------------------

TEMPLATE_URL='ssh://git@gitlab.caishuo.com:10022/docker/docker-template.git'

# 环境变量输入
ask_env(){
  ask_project_env
  ask_global_env
  echo ''
}

ask_global_env() {
  if ( [ "$gitlab_token" = '' ] ); then
    for (( ; ; )); do
      read -p "Gitlab Token:(必填): " gitlab_token
      if ( [ "$gitlab_token" != '' ] ); then
        break
      fi
    done
  else
    echo "Gitlab Token(必填): $gitlab_token"
  fi
}

ask_project_env() {
  if ( [ "$image" = '' ] ); then
    for (( ; ; )); do
      read -p "镜像名称(${image:-必填}): " image
      if ( [ "$image" != '' ] ); then
        break
      fi
    done
  else
    echo "镜像名称(必填): $image"
  fi
  
  read -p "用户名($(default_username)): " username
  read -p "镜像简介(生成README): " intro
}

save_global_env() {
  echo -e "\
gitlab_api_endpoint=$(gitlab_api_endpoint)\n\
gitlab_token=${gitlab_token}\n\
docker_host=${docker_host}\n\
email=${email}\
" > ~/.docker-runner
}

save_project_env() {
  echo -e "\
username=${username}\n\
image=${image}\n\
version=${version}\
" > .docker-runner

  echo -e "\n[本地.docker-runner文件创建成功]"
  cat .docker-runner
}

print_env() {
  print_global_env && echo ''
  print_project_env
}

print_global_env() {
  if require_global_env; then
    echo "全局变量:"
    cat $HOME/.docker-runner
  fi
}

print_project_env() {
  if require_project_env; then
    echo "项目变量:"
    cat .docker-runner
  fi
}

# 重置全局变量
reset_global_env() {
  ask_global_env
  set_default_global_env
  save_global_env
  print_global_env
}

require_global_env() {
  if [ -r "$HOME/.docker-runner" ]; then
    return 0
  else
    echo "**请先执行docker-runner setup, 进行环境初始化"
    exit 11
  fi
}

require_project_env() {
  if require_global_env && [ -r "$PWD/.docker-runner" ]; then
    return 0
  else
    echo "**当前目录不是镜像项目, 请切换至镜像项目"
    exit 12
  fi
}

#------------------------------------------------------------------------------
# 2. 环境变量默认值处理:
#------------------------------------------------------------------------------

# 设置环境变量默认值
# column                默认值
# -------------------+-------------------
# username              有
# image                 必填
# version               latest
# intro                 TODO
# gitlab_domain         gitlab.caishuo.com
# gitlab_token          必填
# docker_host           hub.caishuo.com
set_default_env() {
  set_default_project_env
  set_default_global_env
  save_global_env
}

set_default_global_env() {
  echo -e "全局变量:"
  echo " - Docker Email: ${email:=$(default_email)}"
  echo " - Gitlab Domain: ${gitlab_domain:=$(default_gitlab_domain)}"
  echo " - Gitlab Token: ${gitlab_token}"
  echo " - Docker Host: ${docker_host:=$(default_docker_host)}"
  echo ''
}

set_default_project_env() {
  echo -e "项目变量:"
  echo " - 用户名: ${username:=$(default_username)}"
  echo " - 镜像名称: ${image}"
  echo " - 版本号: ${version:=latest}"
  echo " - 镜像简介: ${intro:=TODO}"
  echo ''
}

# 默认邮箱
default_email() {
  git config user.email
}

# 默认用户名
default_username() {
  local email=$(default_email)
  echo ${email%%@*}
}

default_intro() {
  echo ${intro}
}

default_gitlab_domain() {
  echo 'gitlab.caishuo.com'
}

gitlab_api_endpoint() {
  local domain=$(default_gitlab_domain)
  echo "https://${domain}/api/v3"
}

default_docker_host() {
  echo 'hub.caishuo.com'
}

#------------------------------------------------------------------------------
# 3. git & gitlab & template
#------------------------------------------------------------------------------

clone_template() {
  git clone $TEMPLATE_URL $image && \
  cd $image && \
  rm -rf .git && \
  echo "[模版下载成功]"
}

update_template() {
  echo "模版更新: 当前代码分支[$(git rev-parse --abbrev-ref HEAD)], 即将拉取远程模版代码"
  read -p "确认是否开始更新: (Y/n)      " update_template
  if ([ "$update_template" = '' ] || [ "$update_template" = 'Y' ] || [ "$update_template" = 'y' ]); then
    git remote add upstream $TEMPLATE_URL && \
    git fetch upstream && \
    git merge upstream/master
  else
    echo 停止模版更新
  fi
}

create_gitlab_project() {
  (GITLAB_API_ENDPOINT="$(gitlab_api_endpoint)" \
  GITLAB_API_PRIVATE_TOKEN="$gitlab_token" \
  gitlab create_project $image "{description: $intro, public: false}" &> /dev/null || (echo '[Gitlab项目创建失败]' && exit 1))&& \
  echo '[Gitlab项目创建成功]'
}

add_git() {
  git init
  git remote add origin ssh://git@gitlab.caishuo.com:10022/$username/$image.git &> /dev/null
  git add .
  git commit -am 'init project'
  git push -u origin master
}

#------------------------------------------------------------------------------
# 4. project内部操作
#------------------------------------------------------------------------------

reset_project() {
  save_project_env
  echo ''

  reset_readme
  echo ''

  reset_dockerfile
  echo ''
}


reset_readme() {
  if [ ! -d README.md ] || [ $(grep -c "这个repo是Docker镜像模版" README.md) -gt 0 ] || [ $(cat README.md | wc -c) -eq 0 ]; then
    echo -n -e "# $image\n\n$(default_intro)\n\n\
## 镜像提交:\n\n~~~\n\
docker push hub.caishuo.com/$username/$image\n~~~\n\n
## 执行示例:\n\n~~~\n\
dr hub.caishuo.com/$username/$image /bin/sh\n~~~" > README.md
    echo "[README.md重置成功]"
  else 
    echo "README.md已跳过"
  fi
}

reset_dockerfile() {
  local str="MAINTAINER $username <$(default_email)>"
  if [ $(grep -c "# MAINTAINER" Dockerfile) -gt 0 ]; then
    sed -i '' "s/#.*com>/$str/g" Dockerfile && echo "[Dockerfile设置成功]"
  else
    echo "Dockerfile已跳过"
  fi
}


#------------------------------------------------------------------------------
# 5. Build & Push
#------------------------------------------------------------------------------

README_PATH='README.md'
VALID_STR='这个repo是Docker镜像模版'
BUILD_ERR='请修改README.md, 编译已停止'
OFFICIAL_USER='docker'


error(){
  echo **$@
}

valid_readme() {
  if [ ! -r $README_PATH ]; then
    read -p "README.md不存在, 是否生成?    (Y/n)" is_reset_readme

    if [ "$is_reset_readme" = 'n' ]; then
      error "README.md不存在, 停止构建镜像"
      return 1
    else
      reset_readme
      return 0
    fi
    
  elif [ `grep -c "$VALID_STR" $README_PATH` -gt 0 ]; then
    error $BUILD_ERR && \
    open $README_PATH &&
    return 1
  else
    return 0
  fi
}

valid_variables() {
  local result=0
  if [ ! -n "$username" ] ;then
    error '请在.docker-runner文件中配置username' && result=1
  fi

  if [ ! -n "$image" ];then
    error '请在.docker-runner文件中配置image' && result=1
  fi

  return $result
}

set_image_name() {
  if ! require_project_env; then
    echo '**当前目录不是镜像项目, 请切换至镜像项目'
    exit 3
  fi

  if [ $username = $OFFICIAL_USER ]; then
    img="$docker_host/$image:${version:-latest}"
  else
    img="$docker_host/$username/$image:${version:-latest}"
  fi
}

build_image() {
  set_image_name && \
  (docker build $docker_opts -t $img .) && \
  echo $img was built successfully  
}

push_image() {
  set_image_name
  (docker push $docker_opts $img) && \
  echo $img was pushed successfully  
}

#------------------------------------------------------------------------------
# 6. Docker Registry Login & Logout & Tools
#------------------------------------------------------------------------------

docker_login() {
  echo "正在登录$docker_host:"
  read -p "用户名($(default_username)): " username
  echo "docker login -u ${username:=$(default_username)} -e $email $docker_host"
  docker login -u ${username:=$(default_username)} -e $email $docker_host
}

docker_logout() {
  docker logout $docker_host
}

docker_console() {
  set_image_name && \
  docker run -i -t --rm --name ${NAME:-tmp} $docker_opts $img /bin/sh
}

docker_clean() {
  echo '[开始清理container]' && docker rm `docker ps -a | grep Exited | awk '{print $1 }'`
  echo '[开始清理image]' && docker rmi $(docker images | grep "^<none>" | awk "{print $3}")
  echo '[开始清理volume]' && docker volume rm $(docker volume ls -qf dangling=true)
}

#------------------------------------------------------------------------------
# Public API
#------------------------------------------------------------------------------

# 创建新项目
do_new_project() {
  ask_env && \
  set_default_env && \
  clone_template && \
  reset_project && \
  create_gitlab_project && \
  add_git && \
  echo -e '\n[镜像创建成功]'
}

do_setup() {
  reset_global_env
  do_login
}

do_reset() {
  echo "该操作将删除已有配置"
  read -p "确认是否进行: (Y/n)      " is_reset_env
  if ([ "$is_reset_env" = '' ] || [ "$is_reset_env" = 'Y' ] || [ "$is_reset_env" = 'y' ]); then
    rm -f "$HOME/.docker-runner"
    unset gitlab_token
    echo "清理成功, 开始重置..."
    do_setup
    echo "重置成功"
  else
    echo 停止执行环境重置
  fi
}

do_config() {
  print_env
}

do_build() {
  valid_readme &&\
  valid_variables && build_image
}

do_push() {
  valid_readme &&\
  valid_variables && push_image
}

do_login() {
  docker_login
}

do_logout() {
  docker_logout
}

do_console() {
  docker_console
}

do_clean() {
  docker_clean
}

do_update() {
  update_template
}
